@@ -51,6 +51,7 @@ typedef struct {
 	char *id;
 	char *uri;
 	char *display_name;
+	char *display_name_key;
 	char *icon;
 } GnomeVFSConnectedServer;
 
@@ -638,7 +639,7 @@ create_drive_from_mount_point (GnomeVFSVolumeMonitor *volume_monitor,
 {
 	GnomeVFSDrive *drive;
 	GnomeVFSVolume *mounted_volume;
-	char *uri;
+	char *name, *uri;
 	
 	if (mount->is_loopback ||
 	    !(mount->is_user_mountable ||
@@ -694,6 +695,10 @@ create_drive_from_mount_point (GnomeVFSVolumeMonitor *volume_monitor,
 	drive->priv->icon = get_drive_icon_from_type (drive->priv->device_type, mount->mount_path);
 
 	drive->priv->display_name = get_drive_name (volume_monitor, drive, mount);
+
+	name = g_utf8_casefold (drive->priv->display_name, -1);
+	drive->priv->display_name_key = g_utf8_collate_key (name, -1);
+	g_free (name);
 	
 	drive->priv->is_user_visible = TRUE;
 	drive->priv->volumes = NULL;
@@ -959,7 +964,11 @@ create_vol_from_mount (GnomeVFSVolumeMonitor *volume_monitor, GnomeVFSUnixMount
 	vol->priv->display_name = _gnome_vfs_volume_monitor_uniquify_volume_name (volume_monitor, utf8_name);
 	g_free (display_name);
 	g_free (utf8_name);
-	
+
+	display_name = g_utf8_casefold (vol->priv->display_name, -1);
+	vol->priv->display_name_key = g_utf8_collate_key (display_name, -1);
+	g_free (display_name);
+
 	vol->priv->icon = get_icon_from_type (vol->priv->device_type, mount->mount_path);
 
 	vol->priv->is_user_visible = 0;
@@ -1083,6 +1092,7 @@ connected_server_free (GnomeVFSConnectedServer *server)
 	g_free (server->id);
 	g_free (server->uri);
 	g_free (server->display_name);
+	g_free (server->display_name_key);
 	g_free (server->icon);
 	g_free (server);
 }
@@ -1101,7 +1111,7 @@ connected_server_compare (GnomeVFSConnectedServer *a,
 	if (res != 0) {
 		return res;
 	}
-	res = strcmp (a->display_name, b->display_name);
+	res = strcmp (a->display_name_key, b->display_name_key);
 	if (res != 0) {
 		return res;
 	}
@@ -1156,9 +1166,16 @@ get_connected_servers (GnomeVFSVolumeMonitorDaemon *volume_monitor_daemon)
 				g_free (server->icon);
 				g_free (server);
 			} else {
+				char *name;
+
 				if (server->display_name == NULL) {
 					server->display_name = g_strdup (_("Network server"));
 				}
+
+				name = g_utf8_casefold (server->display_name, -1);
+				server->display_name_key = g_utf8_collate_key (name, -1);
+				g_free (name);
+
 				if (server->icon == NULL) {
 					server->icon = g_strdup ("gnome-fs-share");
 				}
@@ -1180,13 +1197,18 @@ create_volume_from_connected_server (GnomeVFSVolumeMonitor *volume_monitor,
 				     GnomeVFSConnectedServer *server)
 {
 	GnomeVFSVolume *vol;
-
+	char *name;
 	
 	vol = g_object_new (GNOME_VFS_TYPE_VOLUME, NULL);
 
 	vol->priv->volume_type = GNOME_VFS_VOLUME_TYPE_CONNECTED_SERVER;
 	vol->priv->activation_uri = g_strdup (server->uri);
 	vol->priv->display_name = _gnome_vfs_volume_monitor_uniquify_volume_name (volume_monitor, server->display_name);
+
+	name = g_utf8_casefold (vol->priv->display_name, -1);
+	vol->priv->display_name_key = g_utf8_collate_key (name, -1);
+	g_free (name);
+
 	vol->priv->icon = g_strdup (server->icon);
 	vol->priv->gconf_id = g_strdup (server->id);
 	vol->priv->is_mounted = 1;
